use serde::{Deserialize, Serialize};
use serde_json::Value;

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct EventResponse {
    pub id: String,
    pub uid: String,
    pub date: String,
    pub name: String,
    pub short_name: String,
    pub season: Season,
    pub week: Week,
    pub competitions: Vec<Competition>,
    pub links: Vec<Link3>,
    pub status: Status2,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Season {
    pub year: i64,
    #[serde(rename = "type")]
    pub type_field: i64,
    pub slug: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Week {
    pub number: i64,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Competition {
    pub id: String,
    pub uid: String,
    pub date: String,
    pub attendance: i64,
    #[serde(rename = "type")]
    pub type_field: Type,
    pub time_valid: bool,
    pub neutral_site: bool,
    pub conference_competition: bool,
    pub play_by_play_available: bool,
    pub recent: bool,
    pub venue: Venue,
    pub competitors: Vec<Competitor>,
    pub notes: Vec<Value>,
    pub status: Status,
    pub broadcasts: Vec<Broadcast>,
    pub leaders: Vec<Leader>,
    pub format: Format,
    pub start_date: String,
    pub broadcast: String,
    pub geo_broadcasts: Vec<GeoBroadcast>,
    pub highlights: Vec<Value>,
    pub headlines: Option<Vec<Headline>>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Type {
    pub id: String,
    pub abbreviation: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Venue {
    pub id: String,
    pub full_name: String,
    pub address: Address,
    pub indoor: bool,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Address {
    pub city: String,
    pub state: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Competitor {
    pub id: String,
    pub uid: String,
    #[serde(rename = "type")]
    pub type_field: String,
    pub order: i64,
    pub home_away: String,
    pub winner: Option<bool>,
    pub team: Team,
    pub score: String,
    pub linescores: Option<Vec<Linescore>>,
    pub statistics: Vec<Value>,
    pub records: Vec<Record>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Team {
    pub id: String,
    pub uid: String,
    pub location: String,
    pub name: String,
    pub abbreviation: String,
    pub display_name: String,
    pub short_display_name: String,
    pub color: String,
    pub alternate_color: String,
    pub is_active: bool,
    pub venue: Venue2,
    pub links: Vec<Link>,
    pub logo: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Venue2 {
    pub id: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Link {
    pub rel: Vec<String>,
    pub href: String,
    pub text: String,
    pub is_external: bool,
    pub is_premium: bool,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Linescore {
    pub value: f64,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Record {
    pub name: String,
    pub abbreviation: Option<String>,
    #[serde(rename = "type")]
    pub type_field: String,
    pub summary: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Status {
    pub clock: f64,
    pub display_clock: String,
    pub period: i64,
    #[serde(rename = "type")]
    pub type_field: Type2,
    #[serde(rename = "isTBDFlex")]
    pub is_tbdflex: bool,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Type2 {
    pub id: String,
    pub name: String,
    pub state: String,
    pub completed: bool,
    pub description: String,
    pub detail: String,
    pub short_detail: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Broadcast {
    pub market: String,
    pub names: Vec<String>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Leader {
    pub name: String,
    pub display_name: String,
    pub short_display_name: String,
    pub abbreviation: String,
    pub leaders: Vec<Leader2>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Leader2 {
    pub display_value: String,
    pub value: f64,
    pub athlete: Athlete,
    pub team: Team3,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Athlete {
    pub id: String,
    pub full_name: String,
    pub display_name: String,
    pub short_name: String,
    pub links: Vec<Link2>,
    pub headshot: String,
    pub jersey: String,
    pub position: Position,
    pub team: Team2,
    pub active: bool,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Link2 {
    pub rel: Vec<String>,
    pub href: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Position {
    pub abbreviation: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Team2 {
    pub id: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Team3 {
    pub id: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Format {
    pub regulation: Regulation,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Regulation {
    pub periods: i64,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct GeoBroadcast {
    #[serde(rename = "type")]
    pub type_field: Type3,
    pub market: Market,
    pub media: Media,
    pub lang: String,
    pub region: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Type3 {
    pub id: String,
    pub short_name: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Market {
    pub id: String,
    #[serde(rename = "type")]
    pub type_field: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Media {
    pub short_name: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Headline {
    #[serde(rename = "type")]
    pub type_field: String,
    pub description: String,
    pub short_link_text: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Link3 {
    pub language: String,
    pub rel: Vec<String>,
    pub href: String,
    pub text: String,
    pub short_text: String,
    pub is_external: bool,
    pub is_premium: bool,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Status2 {
    pub clock: f64,
    pub display_clock: String,
    pub period: i64,
    #[serde(rename = "type")]
    pub type_field: Type4,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Type4 {
    pub id: String,
    pub name: String,
    pub state: String,
    pub completed: bool,
    pub description: String,
    pub detail: String,
    pub short_detail: String,
}
